# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.design_electric.components.relay.py is part of the
#       RAMSTK Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Relay Input Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKEntry, RAMSTKFixedPanel


class RelayDesignElectricInputPanel(RAMSTKFixedPanel):
    """Display Relay assessment input attribute data in the RAMSTK Work Book.

    The Relay assessment input view displays all the assessment inputs for
    the selected relay.  This includes, currently, inputs for
    MIL-HDBK-217FN2.  The attributes of a Relay assessment input view are:

    :cvar dict _dic_specifications: dictionary of relay MIL-SPECs.  Key is
        relay subcategory ID; values are lists of specifications.
    :cvar dict _dic_styles: dictionary of relay styles defined in the
        MIL-SPECs.  Key is relay subcategory ID; values are lists of styles.

    :ivar cmbType: select and display the type of relay.
    :ivar cmbLoadType: select and display the type of load the relay is
        switching.
    :ivar cmbContactForm: select and display the form of the relay contacts.
    :ivar cmbContactRating: select and display the rating of the relay
        contacts.
    :ivar cmbApplication: select and display the relay application.
    :ivar cmbConstruction: select and display the relay's method of
        construction.
    :ivar txtCycles: enter and display the number of relay cycles per hour.
    """

    _balanced_armature: str = _("Balanced Armature")
    _dry_reed: str = _("Dry Reed")
    _magnetic_latching: str = _("Magnetic Latching")
    _mechanical_latching: str = _("Mechanical Latching")
    _mercury_wetted: str = _("Mercury Wetted")
    _meter_movement: str = _("Meter Movement")
    _solenoid: str = _("Solenoid")

    # Define private dict class attributes.
    _dic_quality: Dict[int, List[List[str]]] = {
        1: [["S"], ["R"], ["P"], ["M"], ["MIL-C-15305"], [_("Lower")]],
        2: [["MIL-SPEC"], [_("Lower")]],
    }
    # Key is subcategory ID.  Index is type ID.
    _dic_pc_types: Dict[int, List[List[str]]] = {
        1: [
            [_("General Purpose")],
            [_("Contactor, High Current")],
            [_("Latching")],
            [_("Reed")],
            [_("Thermal, Bi-Metal")],
            [_meter_movement],
        ],
        2: [[_("Solid State")], [_("Hybrid and Solid State Time Delay")]],
    }
    # Key is subcategory ID, index is type ID.
    _dic_types: Dict[int, List[List[str]]] = {
        1: [[_("85C Rated")], [_("125C Rated")]],
        2: [[_("Solid State")], [_("Solid State Time Delay")], [_("Hybrid")]],
    }
    # Key is contact rating ID.  Index is application ID.
    _dic_application: Dict[int, List[List[str]]] = {
        1: [[_("Dry Circuit")]],
        2: [
            [_("General Purpose")],
            [_("Sensitive (0 - 100mW)")],
            [_("Polarized")],
            [_("Vibrating Reed")],
            [_("High Speed")],
            [_("Thermal Time Delay")],
            [_("Electronic Time Delay, Non-Thermal")],
            [_magnetic_latching],
        ],
        3: [[_("High Voltage")], [_("Medium Power")]],
        4: [[_("Contactors, High Current")]],
    }
    # First key is contact rating ID, second key is application ID.  Index is
    # construction ID.
    _dic_construction: Dict[int, Dict[int, List[List[str]]]] = {
        1: {
            1: [
                [_("Armature (Long)")],
                [_dry_reed],
                [_mercury_wetted],
                [_magnetic_latching],
                [_balanced_armature],
                [_solenoid],
            ]
        },
        2: {
            1: [[_("Armature (Long)")], [_balanced_armature], [_solenoid]],
            2: [
                [_("Armature (Long and Short)")],
                [_mercury_wetted],
                [_magnetic_latching],
                [_meter_movement],
                [_balanced_armature],
            ],
            3: [[_("Armature (Short)")], [_meter_movement]],
            4: [[_dry_reed], [_mercury_wetted]],
            5: [[_("Armature (Balanced and Short)")], [_dry_reed]],
            6: [[_("Bimetal")]],
            8: [
                [_dry_reed],
                [_mercury_wetted],
                [_balanced_armature],
            ],
        },
        3: {
            1: [[_("Vacuum (Glass)")], [_("Vacuum (Ceramic)")]],
            2: [
                [_("Armature (Long and Short)")],
                [_mercury_wetted],
                [_magnetic_latching],
                [_mechanical_latching],
                [_balanced_armature],
                [_solenoid],
            ],
        },
        4: {
            1: [
                [_("Armature (Short)")],
                [_mechanical_latching],
                [_balanced_armature],
                [_solenoid],
            ]
        },
    }

    # Define private list class attributes.

    # Index is the technology ID (load type).
    _lst_technology: List[List[str]] = [[_("Resistive")], [_("Inductive")], [_("Lamp")]]

    # Index is the contact form ID.
    _lst_contact_form: List[List[str]] = [
        ["SPST"],
        ["DPST"],
        ["SPDT"],
        ["3PST"],
        ["4PST"],
        ["DPDT"],
        ["3PDT"],
        ["4PDT"],
        ["6PDT"],
    ]

    # Index is contact rating ID.
    _lst_contact_rating: List[List[str]] = [
        [_("Signal Current (low mV and mA)")],
        [_("0 - 5 Amp")],
        [_("5 - 20 Amp")],
        [_("20 - 600 Amp")],
    ]

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "selected_hardware"
    _tag: str = "design_electric"
    _title: str = _("Relay Design Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Relay assessment input view."""
        super().__init__()

        # Initialize widgets.
        self.cmbApplication: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbConstruction: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbContactForm: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbContactRating: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbLoadType: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbQuality: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbType: RAMSTKComboBox = RAMSTKComboBox()
        self.txtCycles: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._hazard_rate_method_id: int = 0
        self._quality_id: int = 0

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "quality_id": [
                32,
                self.cmbQuality,
                "changed",
                super().on_changed_combo,
                "wvw_editing_reliability",
                0,
                {
                    "tooltip": _("The quality level."),
                },
                _("Quality Level:"),
                "gint",
            ],
            "type_id": [
                48,
                self.cmbType,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The relay type."),
                },
                _("Type:"),
                "gint",
            ],
            "technology_id": [
                37,
                self.cmbLoadType,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The type of load the relay is switching."),
                },
                _("Load Type:"),
                "gint",
            ],
            "contact_form_id": [
                7,
                self.cmbContactForm,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The contact form of the relay."),
                },
                _("Contact Form:"),
                "gint",
            ],
            "contact_rating_id": [
                9,
                self.cmbContactRating,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The rating of the relay contacts."),
                },
                _("Contact Rating:"),
                "gint",
            ],
            "application_id": [
                2,
                self.cmbApplication,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The type of relay application."),
                },
                _("Application:"),
                "gint",
            ],
            "construction_id": [
                6,
                self.cmbConstruction,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The method of construction of the relay."),
                },
                _("Construction:"),
                "gint",
            ],
            "n_cycles": [
                2,
                self.txtCycles,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The number of relay on/off cycles per hour."),
                },
                _("Number of Cycles/Hour:"),
                "gfloat",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.category_id: int = 0
        self.subcategory_id: int = 0

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.cmbContactRating.connect("changed", self._on_combo_changed, 4)
        self.cmbApplication.connect("changed", self._on_combo_changed, 5)

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self.do_load_comboboxes,
            "changed_subcategory",
        )
        pub.subscribe(
            self._do_set_reliability_attributes,
            "succeed_get_reliability_attributes",
        )

    def do_load_comboboxes(self, subcategory_id: int) -> None:
        """Load the Relay RAMSTKComboBox()s.

        :param subcategory_id: the subcategory ID of the selected relay.
        :return: None
        :rtype: None
        """
        self.subcategory_id = subcategory_id

        self.__do_load_quality_combo()
        self.__do_load_type_combo()

        self.cmbLoadType.do_load_combo(self._lst_technology, signal="changed")
        self.cmbContactForm.do_load_combo(self._lst_contact_form, signal="changed")
        self.cmbContactRating.do_load_combo(self._lst_contact_rating, signal="changed")

    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the reliability attributes are retrieved.

        :param attributes: the dict of reliability attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._hazard_rate_method_id = attributes["hazard_rate_method_id"]
            self._quality_id = attributes["quality_id"]

            self.cmbQuality.set_sensitive(True)
            self.cmbQuality.do_update(
                self._quality_id,
                signal="changed",
            )

            pub.sendMessage(
                f"request_get_{self._tag}_attributes",
                node_id=self._record_id,
            )

    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for the selected relay.

        :return: None
        :rtype: None
        """
        self.cmbType.set_sensitive(True)
        self.cmbType.do_update(
            attributes["type_id"],
            signal="changed",
        )

        self.cmbLoadType.set_sensitive(False)
        self.cmbContactForm.set_sensitive(False)
        self.cmbContactRating.set_sensitive(False)
        self.cmbApplication.set_sensitive(False)
        self.cmbConstruction.set_sensitive(False)
        self.txtCycles.set_sensitive(False)

        if self._hazard_rate_method_id == 2 and self.subcategory_id == 1:
            self.cmbLoadType.set_sensitive(True)
            self.cmbLoadType.do_update(
                attributes["technology_id"],
                signal="changed",
            )

            self.cmbContactForm.set_sensitive(True)
            self.cmbContactForm.do_update(
                attributes["contact_form_id"],
                signal="changed",
            )

            self.cmbContactRating.set_sensitive(True)
            self.cmbContactRating.do_update(
                attributes["contact_rating_id"],
                signal="changed",
            )

            self.cmbApplication.set_sensitive(True)
            self.__do_load_application_combo()
            self.cmbApplication.do_update(
                attributes["application_id"],
                signal="changed",
            )

            self.cmbConstruction.set_sensitive(True)
            self.__do_load_construction_combo()
            self.cmbConstruction.do_update(
                attributes["construction_id"],
                signal="changed",
            )

            self.txtCycles.set_sensitive(True)
            self.txtCycles.do_update(
                str(attributes["n_cycles"]),
                signal="changed",
            )

    def _on_combo_changed(self, __combo: RAMSTKComboBox, index: int) -> None:
        """Retrieve RAMSTKCombo() changes and assign to Relay attribute.

        This method is called by:

            * Gtk.Combo() 'changed' signal

        :param __combo: the RAMSTKCombo() that called this method.  This
            parameter is unused in this method but is required because the
            widget provides it to the callback function.
        :param index: the position in the signal handler list associated
            with the calling RAMSTKComboBox().  Indices are:

            +-------+------------------+-------+------------------+
            | Index | Widget           | Index | Widget           |
            +=======+==================+=======+==================+
            |   4   | cmbContactRating |   5   | cmbApplication   |
            +-------+------------------+-------+------------------+

        :return: None
        :rtype: None
        """
        if index == 4:
            self.__do_load_application_combo()
        elif index == 5:
            self.__do_load_construction_combo()

    def __do_load_application_combo(self) -> None:
        """Load the selections in the Relay application RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        _contact_rating_id = int(self.cmbContactRating.get_active())
        try:
            _data = self._dic_application[_contact_rating_id]
        except KeyError:
            _data = []
        self.cmbApplication.do_load_combo(_data, signal="changed")

    def __do_load_construction_combo(self) -> None:
        """Load the selections in the Relay construction RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        _application_id = int(self.cmbApplication.get_active())
        _contact_rating_id = int(self.cmbContactRating.get_active())
        try:
            _data = self._dic_construction[_contact_rating_id][_application_id]
        except KeyError:
            _data = []
        self.cmbConstruction.do_load_combo(_data, signal="changed")

    def __do_load_quality_combo(self) -> None:
        """Load the selections in the Relay quality RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 1:
            _data = [[_("Established Reliability")], ["MIL-SPEC"], [_("Lower")]]
        else:
            try:
                _data = self._dic_quality[self.subcategory_id]
            except KeyError:
                _data = []
        self.cmbQuality.do_load_combo(_data, signal="changed")

    def __do_load_type_combo(self) -> None:
        """Load the selections in the Relay type RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        try:
            _data = self._dic_types[self.subcategory_id]
        except KeyError:
            _data = []
        self.cmbType.do_load_combo(_data, signal="changed")



# Standard Library Imports
from typing import Any, Dict, Tuple

ACTIVATION_ENERGY: Any
C1: Any
C2: Any
PART_COUNT_LAMBDA_B: Any
PI_A: Any
PI_E: Any
PI_PT: Any
PI_Q: Any

def calculate_die_complexity_factor(area: float, feature_size: float) -> float: ...
def calculate_junction_temperature(
    temperature_case: float, power_operating: float, theta_jc: float
) -> float: ...
def calculate_lambda_cyclic_factors(
    n_cycles: int, construction_id: int, n_elements: int, temperature_junction: float
) -> Tuple[float, float, float, float]: ...
def calculate_temperature_factor(
    subcategory_id: int, family_id: int, type_id: int, temperature_junction: float
) -> float: ...
def calculate_eos_hazard_rate(voltage_esd: float) -> float: ...
def calculate_package_base_hazard_rate(n_active_pins: int) -> float: ...
def calculate_package_factor(package_id: int, n_active_pins: int) -> float: ...
def calculate_part_count(**attributes: Dict[str, Any]) -> float: ...
def calculate_part_stress(**attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def get_application_factor(type_id: int, application_id: int) -> float: ...
def get_die_complexity_factor(
    subcategory_id: int, technology_id: int, application_id: int, n_elements: int
) -> float: ...
def get_die_base_hazard_rate(type_id: int) -> float: ...
def get_error_correction_factor(type_id: int) -> float: ...
def get_manufacturing_process_factor(manufacturing_id: int) -> float: ...
def get_package_type_correction_factor(package_id: int) -> float: ...
def get_part_count_lambda_b(n_elements: int, id_keys: Dict[str, int]) -> float: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.requirement.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Requirement Views."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKEntry,
    RAMSTKFixedPanel,
    RAMSTKMessageDialog,
    RAMSTKModuleView,
    RAMSTKWorkView,
)

# RAMSTK Local Imports
from . import (
    RequirementClarityPanel,
    RequirementCompletenessPanel,
    RequirementConsistencyPanel,
    RequirementGeneralDataPanel,
    RequirementTreePanel,
    RequirementVerifiabilityPanel,
)


class RequirementModuleView(RAMSTKModuleView):
    """Display Requirement attribute data in the RAMSTK Module Book.

    The Requirement Module View displays all the Requirements associated with
    the connected RAMSTK Program in a flat list.  The attributes of a
    Requirement Module View are:

    :cvar _tag: the name of the module.

    :ivar _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "requirement"
    _tablabel: str = "Requirement"
    _tabtooltip: str = _(
        "Displays the requirements hierarchy for the selected Revision."
    )

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Requirement Module View.

        :param configuration: the RAMSTK Configuration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.
        self._dic_icons["tab"] = (
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/requirement.png"
        )

        # Initialize private list attributes.
        self._lst_callbacks.insert(1, self.do_request_insert_child)
        self._lst_icons[0] = "insert_sibling"
        self._lst_icons.insert(1, "insert_child")
        self._lst_mnu_labels = [
            _("Add Sibling Requirement"),
            _("Add Child Requirement"),
            _("Delete Selected Requirement"),
            _("Save Selected Requirement"),
            _("Save All Requirements"),
        ]
        self._lst_tooltips = [
            _("Add a new sibling requirement."),
            _("Add a new child requirement."),
            _("Remove the currently selected requirement."),
            _("Save changes to the currently selected requirement."),
            _("Save changes to all requirements"),
        ]

        # Initialize private scalar attributes.
        self._pnlPanel = RequirementTreePanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, f"selected_{self._tag}")

    def do_request_delete(self, __button: Gtk.ToolButton) -> None:
        """Request to delete selected record from the RAMSTKRequirement table.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        _parent = self.get_parent().get_parent().get_parent().get_parent().get_parent()
        _prompt = _(
            f"You are about to delete Requirement {self.dic_pkeys['record_id']} and "
            f"all data associated with it.  Is this really what you want to do?"
        )
        _dialog = RAMSTKMessageDialog(parent=_parent)
        _dialog.do_set_message(_prompt)
        _dialog.do_set_message_type("question")

        if _dialog.do_run() == Gtk.ResponseType.YES:
            super().do_set_cursor_busy()
            pub.sendMessage(
                "request_delete_requirement", node_id=self.dic_pkeys["record_id"]
            )

        _dialog.do_destroy()

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the Requirement's record and parent ID.

        :param attributes: the attributes dict for the selected Requirement.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["revision_id"] = attributes["revision_id"]
        self.dic_pkeys["parent_id"] = attributes["parent_id"]
        self.dic_pkeys["record_id"] = attributes["requirement_id"]
        self.dic_pkeys["requirement_id"] = attributes["requirement_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the requirement module view.

        :return: None
        """
        super().make_ui()

        for (__, _key) in enumerate(  # pylint: disable=unused-variable
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_WORKGROUPS
        ):
            self._pnlPanel.lst_owner.append(
                self.RAMSTK_USER_CONFIGURATION.RAMSTK_WORKGROUPS[_key][0]
            )

        for (__, _key) in enumerate(  # pylint: disable=unused-variable
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_REQUIREMENT_TYPE
        ):
            self._pnlPanel.lst_type.append(
                self.RAMSTK_USER_CONFIGURATION.RAMSTK_REQUIREMENT_TYPE[_key][1]
            )

        # pylint: disable=unused-variable
        self._pnlPanel.tvwTreeView.do_load_combo_cell(
            self._pnlPanel.tvwTreeView.position["owner"],
            self._pnlPanel.lst_owner,
        )
        self._pnlPanel.tvwTreeView.do_load_combo_cell(
            self._pnlPanel.tvwTreeView.position["priority"],
            ["", "1", "2", "3", "4", "5"],
        )
        self._pnlPanel.tvwTreeView.do_load_combo_cell(
            self._pnlPanel.tvwTreeView.position["requirement_type"],
            self._pnlPanel.lst_type,
        )

        self._pnlPanel.do_set_cell_callbacks(
            "mvw_editing_requirement",
            [
                "derived",
                "description",
                "figure_number",
                "owner",
                "page_number",
                "priority",
                "specification",
                "requirement_type",
                "validated",
                "validated_date",
            ],
        )
        self._pnlPanel.tvwTreeView.dic_handler_id[
            "button-press"
        ] = self._pnlPanel.tvwTreeView.connect(
            "button_press_event", super().on_button_press
        )


class RequirementGeneralDataView(RAMSTKWorkView):
    """Display general Requirement attribute data in the RAMSTK Work Book.

    The Requirement Work View displays all the general data attributes for the
    selected Requirement.  The attributes of a requirement Work View are:

    :cvar str _tag: the name of the module.
    :cvar str _tablabel: the text to display on the tab's label.
    :cvar str _tabtooltip: the text to display as the tab's tooltip.

    :ivar list _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar list _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar list _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar list _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "requirement"
    _tablabel: str = _("General\nData")
    _tabtooltip: str = _("Displays general information for the selected Requirement.")

    # Define public dict class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Requirement Work View general data page.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :type configuration:
            :class:`ramstk.configuration.RAMSTKUserConfiguration`
        :param logger: the RAMSTKLogManager class instance.
        :type logger: :class:`ramstk.logger.RAMSTKLogManager`
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.
        self._dic_icons["create_code"] = (
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/create_code.png"
        )

        # Initialize private list attributes.
        self._lst_callbacks.insert(0, self._do_request_create_code)
        self._lst_icons.insert(0, "create_code")
        self._lst_mnu_labels = [
            _("Create Code"),
            _("Save Selected Requirement"),
            _("Save All Requirements"),
        ]
        self._lst_tooltips = [
            _("Automatically create code for the selected requirement."),
            _("Save changes to the currently selected requirement."),
            _("Save changes to all requirements."),
        ]

        # Initialize private scalar attributes.
        self._pnlGeneralData: RAMSTKFixedPanel = RequirementGeneralDataPanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_set_cursor_active, "succeed_create_code")

        pub.subscribe(self._do_set_record_id, "selected_requirement")

    def _do_request_create_code(self, __button: Gtk.ToolButton) -> None:
        """Request that requirement codes be built.

        :param __button: the Gtk.ToolButton() that called this method.
        :type __button: :py:class:`Gtk.ToolButton`
        :return: None
        :rtype: None
        """
        _prefix = self._pnlGeneralData.cmbRequirementType.get_value()

        super().do_set_cursor_busy()
        pub.sendMessage(
            "request_create_requirement_code",
            node_id=self.dic_pkeys["record_id"],
            prefix=_prefix,
        )

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the record and parent ID.

        :param attributes: the attributes dict for the selected requirement.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["revision_id"] = attributes["revision_id"]
        self.dic_pkeys["parent_id"] = attributes["parent_id"]
        self.dic_pkeys["record_id"] = attributes["requirement_id"]
        self.dic_pkeys["requirement_id"] = attributes["requirement_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the Requirement General Data tab.

        :return: None
        :rtype: None
        """
        super().do_make_layout()

        # Add the validation date dialog launcher button to the right of the
        # validated date RAMSTKEntry.
        _fixed: Gtk.Fixed = (
            self._pnlGeneralData.get_children()[0].get_children()[0].get_children()[0]
        )
        _entry: RAMSTKEntry = _fixed.get_children()[-1]
        _x_pos: int = _fixed.child_get_property(_entry, "x") + 205
        _y_pos: int = _fixed.child_get_property(_entry, "y")
        _fixed.put(self._pnlGeneralData.btnValidateDate, _x_pos, _y_pos)

        self._pnlGeneralData.do_load_priorities()
        self._pnlGeneralData.do_load_requirement_types(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_REQUIREMENT_TYPE
        )
        self._pnlGeneralData.do_load_workgroups(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_WORKGROUPS
        )

        self.pack_end(self._pnlGeneralData, True, True, 0)
        self.show_all()


class RequirementAnalysisView(RAMSTKWorkView):
    """Display Requirement attribute data in the RAMSTK Work Book.

    The Requirement Analysis Work View displays all the analysis questions and
    answers for the selected Requirement. The attributes of a Requirement
    Analysis Work View are:

    :cvar str _tag: the name of the module.
    :cvar str _tablabel: the text to display on the tab's label.
    :cvar str _tabtooltip: the text to display as the tab's tooltip.

    :ivar list _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar list _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar list _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar list _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dict class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "requirement"
    _tablabel: str = "<span weight='bold'>" + _("Analysis") + "</span>"
    _tabtooltip: str = _("Analyzes the selected requirement.")

    # Define public dict class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Requirements analysis work view.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :type configuration:
            :class:`ramstk.configuration.RAMSTKUserConfiguration`
        :param logger: the RAMSTKLogManager class instance.
        :type logger: :class:`ramstk.logger.RAMSTKLogManager`
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._pnlClarity: RAMSTKFixedPanel = RequirementClarityPanel()
        self._pnlCompleteness: RAMSTKFixedPanel = RequirementCompletenessPanel()
        self._pnlConsistency: RAMSTKFixedPanel = RequirementConsistencyPanel()
        self._pnlVerifiability: RAMSTKFixedPanel = RequirementVerifiabilityPanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_requirement")

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the record and parent ID.

        :param attributes: the attributes dict for the selected requirement.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["revision_id"] = attributes["revision_id"]
        self.dic_pkeys["parent_id"] = attributes["parent_id"]
        self.dic_pkeys["record_id"] = attributes["requirement_id"]
        self.dic_pkeys["requirement_id"] = attributes["requirement_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the Requirement Analysis tab.

        :return: None
        :rtype: None
        """
        _vpaned_left, _vpaned_right = super().do_make_layout_llrr()

        _vpaned_left.pack1(self._pnlClarity, False)
        _vpaned_left.pack2(self._pnlCompleteness, False)
        _vpaned_right.pack1(self._pnlConsistency, False)
        _vpaned_right.pack2(self._pnlVerifiability, False)

        self.show_all()



# type: ignore
# -*- coding: utf-8 -*-
#
#       ramstk.analyses.prediction.Inductor.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Inductor MIL-HDBK-217F Constants and Calculations Module."""

# Standard Library Imports
from math import exp
from typing import Any, Dict, List

PART_COUNT_LAMBDA_B = {
    1: {
        1: [
            0.0035,
            0.023,
            0.049,
            0.019,
            0.065,
            0.027,
            0.037,
            0.041,
            0.052,
            0.11,
            0.0018,
            0.053,
            0.16,
            2.3,
        ],
        2: [
            0.0071,
            0.046,
            0.097,
            0.038,
            0.13,
            0.055,
            0.073,
            0.081,
            0.10,
            0.22,
            0.035,
            0.11,
            0.31,
            4.7,
        ],
        3: [
            0.023,
            0.16,
            0.35,
            0.13,
            0.45,
            0.21,
            0.27,
            0.35,
            0.45,
            0.82,
            0.011,
            0.37,
            1.2,
            16.0,
        ],
        4: [
            0.028,
            0.18,
            0.39,
            0.15,
            0.52,
            0.22,
            0.29,
            0.33,
            0.42,
            0.88,
            0.015,
            0.42,
            1.2,
            19.0,
        ],
    },
    2: {
        1: [
            0.0017,
            0.0073,
            0.023,
            0.0091,
            0.031,
            0.011,
            0.015,
            0.016,
            0.022,
            0.052,
            0.00083,
            0.25,
            0.073,
            1.1,
        ],
        2: [
            0.0033,
            0.015,
            0.046,
            0.018,
            0.061,
            0.022,
            0.03,
            0.033,
            0.044,
            0.10,
            0.0017,
            0.05,
            0.15,
            2.2,
        ],
    },
}
PART_COUNT_PI_Q = [0.25, 1.0, 10.0]
PART_STRESS_PI_Q = {
    1: {1: [1.5, 5.0], 2: [3.0, 7.5], 3: [8.0, 30.0], 4: [12.0, 30.0]},
    2: [0.03, 0.1, 0.3, 1.0, 4.0, 20.0],
}
PI_E = {
    1: [1.0, 6.0, 12.0, 5.0, 16.0, 6.0, 8.0, 7.0, 9.0, 24.0, 0.5, 13.0, 34.0, 610.0],
    2: [1.0, 4.0, 12.0, 5.0, 16.0, 5.0, 7.0, 6.0, 8.0, 24.0, 0.5, 13.0, 34.0, 610.0],
}
REF_TEMPS = {
    1: {1: 329.0, 2: 352.0, 3: 364.0, 4: 400.0, 5: 398.0, 6: 477.0},
    2: {1: 329.0, 2: 352.0, 3: 364.0, 4: 409.0},
}


def calculate_hot_spot_temperature(
    temperature_active: float, temperature_rise: float
) -> float:
    """Calculate the coil or transformer hot spot temperature.

    :return: _temperature_hot_spot; the calculate hot spot temperature.
    :rtype: float
    """
    return temperature_active + 1.1 * temperature_rise


def calculate_part_count(**attributes: Dict[str, Any]) -> float:
    """Wrap get_part_count_lambda_b().

    This wrapper allows us to pass an attributes dict from a generic parts
    count function.

    :param attributes: the attributes for the connection being calculated.
    :return: _base_hr; the parts count base hazard rates.
    :rtype: float
    """
    return get_part_count_lambda_b(
        id_keys={
            "subcategory_id": attributes["subcategory_id"],
            "family_id": attributes["family_id"],
            "environment_active_id": attributes["environment_active_id"],
        }
    )


def calculate_part_stress(**attributes: Dict[str, Any]) -> Dict[str, Any]:
    """Calculate the part stress hazard rate for a inductor.

    This function calculates the MIL-HDBK-217F hazard rate using the part
    stress method.

    :return: attributes; the keyword argument (hardware attribute)
        dictionary with updated values.
    :rtype: dict
    """
    attributes["piC"] = float(attributes["construction_id"])
    attributes["piQ"] = get_part_stress_quality_factor(
        attributes["subcategory_id"], attributes["quality_id"], attributes["family_id"]
    )

    _power_input = attributes["voltage_dc_operating"] * attributes["current_operating"]
    if attributes["subcategory_id"] == 2 and attributes["specification_id"] == 2:
        attributes["temperature_rise"] = get_temperature_rise_spec_sheet(
            attributes["page_number"]
        )
    elif attributes["power_operating"] > 0.0 and attributes["area"] > 0.0:
        attributes["temperature_rise"] = calculate_temperature_rise_power_loss_surface(
            attributes["power_operating"], attributes["area"]
        )
    elif attributes["power_operating"] > 0.0 and attributes["weight"] > 0.0:
        attributes["temperature_rise"] = calculate_temperature_rise_power_loss_weight(
            attributes["power_operating"], attributes["weight"]
        )
    elif _power_input > 0.0 and attributes["weight"] > 0.0:
        attributes["temperature_rise"] = calculate_temperature_rise_input_power_weight(
            _power_input, attributes["weight"]
        )
    else:
        attributes["temperature_rise"] = 0.0
    attributes["temperature_hot_spot"] = calculate_hot_spot_temperature(
        attributes["temperature_active"], attributes["temperature_rise"]
    )
    attributes["lambda_b"] = calculate_part_stress_lambda_b(
        attributes["subcategory_id"],
        attributes["insulation_id"],
        attributes["temperature_hot_spot"],
    )

    attributes["hazard_rate_active"] = (
        attributes["lambda_b"] * attributes["piQ"] * attributes["piE"]
    )
    if attributes["subcategory_id"] == 2:
        attributes["hazard_rate_active"] = (
            attributes["hazard_rate_active"] * attributes["piC"]
        )

    return attributes


def calculate_part_stress_lambda_b(
    subcategory_id: int, insulation_id: int, temperature_hot_spot: float
) -> float:
    """Calculate part stress base hazard rate (lambda b) from MIL-HDBK-217F.

    This function calculates the MIL-HDBK-217F hazard rate using the parts
    stress method.

    :param attributes: the attributes for the connection being calculated.
    :return: _lambda_b; the calculate parts stress lambda_b.
    :rtype: float
    :raise: KeyError when passed an unknown subcategory ID or insulation ID.
    """
    _dic_factors = {
        1: {
            1: [0.0018, 15.6],
            2: [0.002, 14.0],
            3: [0.0018, 8.7],
            4: [0.002, 10.0],
            5: [0.00125, 3.8],
            6: [0.00159, 8.4],
        },
        2: {
            1: [0.000335, 15.6],
            2: [0.000379, 14.0],
            3: [0.000319, 8.7],
            4: [0.00035, 10.0],
        },
    }

    _ref_temp = REF_TEMPS[subcategory_id][insulation_id]
    _f0 = _dic_factors[subcategory_id][insulation_id][0]
    _f1 = _dic_factors[subcategory_id][insulation_id][1]
    _lambda_b = _f0 * exp(((temperature_hot_spot + 273.0) / _ref_temp) ** _f1)

    return _lambda_b


def calculate_temperature_rise_input_power_weight(
    power_input: float, weight: float
) -> float:
    """Calculate the temperature rise based on input power and xfmr weight.

    .. attention:: input power must be calculated by the calling function from
    voltage and current as it is not an attribute of an inductive device.

    :param power_input: the input power in W.
    :param weight: the weight of the xfmr in lbf.
    :retur: _temperature_rise; the calculated temperature rise in C.
    :rtype: float
    :raise: ZeroDivisionError if passed an weight=0.0.
    """
    return 2.1 * (power_input / weight ** 0.6766)


def calculate_temperature_rise_power_loss_surface(
    power_operating: float, area: float
) -> float:
    """Calculate the temperature rise based on the power loss and surface area.

    :param power_operating: the power loss in W.
    :param area: the radiating surface area of the case in sq. inches.
    :return: _temperature_rise; the calculated temperature rise in C.
    :rtype: float
    :raise: ZeroDivisionError if passed an area=0.0.
    """
    return 125.0 * power_operating / area


def calculate_temperature_rise_power_loss_weight(
    power_operating: float, weight: float
) -> float:
    """Calculate the temperature rise based on the power loss and xfmr weight.

    :param power_operating: the power loss in W.
    :param weight: the weight of the device in lbf.
    :return: _temperature_rise; the calculated temperature rise in C.
    :rtype: float
    :raise: ZeroDivisionError if passed an weight=0.0.
    """
    return 11.5 * (power_operating / weight ** 0.6766)


def get_part_count_lambda_b(id_keys: Dict[str, int]) -> List[float]:
    """Retrieve the parts count base hazard rate (lambda b) from MIL-HDBK-217F.

    This function calculates the MIL-HDBK-217F hazard rate using the parts
    count method.

    This function calculates the MIL-HDBK-217F hazard rate using the parts
    count method.  The dictionary PART_COUNT_217F_LAMBDA_B contains the
    MIL-HDBK-217F parts count base hazard rates.  Keys are for
    PART_COUNT_217F_LAMBDA_B are:

        #. subcategory_id
        #. environment_active_id
        #. family id; if the inductor subcategory is NOT family dependent, then
            the second key will be zero.

    Current subcategory IDs are:

    +----------------+-------------------------------+-----------------+
    | Subcategory    |           Inductor            | MIL-HDBK-217F   |
    |       ID       |             Style             |    Section      |
    +================+===============================+=================+
    |        1       | Transformer                   |       11.1      |
    +----------------+-------------------------------+-----------------+
    |        2       | Coil                          |       11.2      |
    +----------------+-------------------------------+-----------------+

    These keys return a list of base hazard rates.  The hazard rate to use is
    selected from the list depending on the active environment.

    :param id_keys: the ID's used as keys when selecting
        the base hazard rate.  The keys are subcategory_id,
        environment_active_id, and family_id.
    :return: _base_hr; the list of part count base hazard rate.
    :rtype: list
    :raise: KeyError if passed an unknown subcategory ID or family ID.
    :raise: IndexError if passed an unknown active environment ID.
    """
    return PART_COUNT_LAMBDA_B[id_keys["subcategory_id"]][id_keys["family_id"]][
        id_keys["environment_active_id"] - 1
    ]


def get_part_stress_quality_factor(
    subcategory_id: int, quality_id: int, family_id: int
) -> float:
    """Select the MIL-HDBK-217F quality factor for the inductor device.

    :param subcategory_id: the subcategory identifier.
    :param quality_id: the quality level identifier.
    :param family_id: the device family identifier.
    :return: _pi_q; the selected quality factor
    :rtype: float
    :raise: IndexError if passed an unknown quality ID.
    :raise: KeyError if passed an unknown subcategory ID or family ID.
    """
    if subcategory_id == 1:
        _pi_q = PART_STRESS_PI_Q[subcategory_id][family_id][quality_id - 1]
    else:
        _pi_q = PART_STRESS_PI_Q[subcategory_id][quality_id - 1]

    return _pi_q


def get_temperature_rise_spec_sheet(page_number: int) -> float:
    """Retrieve the temperature rise based on the spec sheet from MIL-C-39010.

    :param page_number: the spec sheet to retrieve the temperature rise
        for.
    :return: _temperature_rise; the spec sheet temperature rise.
    :rtype: float
    :raise: KeyError if an unknown spec sheet is passed.
    """
    return {
        1: 15.0,
        2: 15.0,
        3: 15.0,
        4: 35.0,
        5: 15.0,
        6: 35.0,
        7: 15.0,
        8: 35.0,
        9: 15.0,
        10: 15.0,
        11: 35.0,
        12: 35.0,
        13: 15.0,
        14: 15.0,
    }[page_number]



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.validation.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Validation Views."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKMessageDialog,
    RAMSTKModuleView,
    RAMSTKPanel,
    RAMSTKWorkView,
)

# RAMSTK Local Imports
from . import (
    ValidationTaskDescriptionPanel,
    ValidationTaskEffortPanel,
    ValidationTreePanel,
)


class ValidationModuleView(RAMSTKModuleView):
    """Display Validation attribute data in the RAMSTK Module Book.

    The Validation Module View displays all the Validations associated with the
    connected RAMSTK Program in a flat list.  The attributes of a Validation
    Module View are:

    :cvar _tag: the name of the module.

    :ivar _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "validation"
    _tablabel: str = "Verification"
    _tabtooltip: str = _(
        "Displays the list of verification tasks for the selected Revision."
    )

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Validation Module View.

        :param configuration: the RAMSTK Configuration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.
        self._dic_icons["tab"] = (
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_ICON_DIR + "/32x32/validation.png"
        )

        # Initialize private list attributes.
        self._lst_mnu_labels = [
            _("Add Verification Task"),
            _("Delete Selected Task"),
            _("Save Selected Task"),
            _("Save All Tasks"),
        ]
        self._lst_tooltips = [
            _("Add a new verification task."),
            _("Remove the currently selected verification task."),
            _("Save changes to the currently selected verification task."),
            _("Save changes to all verification tasks."),
        ]

        # Initialize private scalar attributes.
        self._pnlPanel = ValidationTreePanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, f"selected_{self._tag}")

    def do_request_delete(self, __button: Gtk.ToolButton) -> None:
        """Request to delete selected record from the RAMSTKValidation table.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        """
        _parent = self.get_parent().get_parent().get_parent().get_parent().get_parent()
        _prompt = _(
            "You are about to delete Validation {0:d} and all "
            "data associated with it.  Is this really what "
            "you want to do?"
        ).format(self.dic_pkeys["record_id"])
        _dialog: RAMSTKMessageDialog = RAMSTKMessageDialog(parent=_parent)
        _dialog.do_set_message(_prompt)
        _dialog.do_set_message_type("question")

        if _dialog.do_run() == Gtk.ResponseType.YES:
            super().do_set_cursor_busy()
            pub.sendMessage(
                "request_delete_validation", node_id=self.dic_pkeys["record_id"]
            )

        _dialog.do_destroy()

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the Verification task's record ID.

        :param attributes: the attributes dict for the selected Verification
            task.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["record_id"] = attributes["validation_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the requirement module view.

        :return: None
        """
        super().make_ui()

        self._pnlPanel.do_load_measurement_units(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_MEASUREMENT_UNITS
        )
        self._pnlPanel.do_load_verification_types(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_VALIDATION_TYPE
        )

        self._pnlPanel.do_set_cell_callbacks(
            "mvw_editing_validation",
            [
                "acceptable_maximum",
                "acceptable_mean",
                "acceptable_minimum",
                "acceptable_variance",
                "confidence",
                "cost_average",
                "cost_maximum",
                "cost_minimum",
                "date_end",
                "date_start",
                "description",
                "measurement_unit",
                "name",
                "status",
                "task_specification",
                "task_type",
                "time_average",
                "time_maximum",
                "time_minimum",
            ],
        )
        self._pnlPanel.tvwTreeView.dic_handler_id[
            "button-press"
        ] = self._pnlPanel.tvwTreeView.connect(
            "button_press_event", super().on_button_press
        )


class ValidationGeneralDataView(RAMSTKWorkView):
    """Display general Validation attribute data in the RAMSTK Work Book.

    The Validation Work View displays all the general data attributes for the
    selected Validation. The attributes of a Validation General Data Work View
    are:

    :cvar dict _dic_keys:
    :cvar list _lst_labels: the list of label text.
    :cvar str _tag: the name of the module.

    :ivar list _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar list _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar list _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar list _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "validation"
    _tablabel: str = _("General\nData")
    _tabtooltip: str = _(
        "Displays general information for the selected Verification task."
    )

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Validation Work View general data page.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_callbacks = [
            self._do_request_calculate,
            self._do_request_calculate_all,
            super().do_request_update,
            super().do_request_update_all,
        ]
        self._lst_icons = ["calculate", "calculate_all", "save", "save-all"]
        self._lst_mnu_labels = [
            _("Calculate Task"),
            _("Calculate Program"),
            _("Save"),
            _("Save All"),
        ]
        self._lst_tooltips = [
            _(
                "Calculate the expected cost and time of the selected "
                "Validation task."
            ),
            _(
                "Calculate the cost and time of the program (i.e., all "
                "Validation tasks)."
            ),
            _("Save changes to the selected Validation task."),
            _("Save changes to all Validation tasks."),
        ]

        # Initialize private scalar attributes.
        self._pnlTaskDescription: RAMSTKPanel = ValidationTaskDescriptionPanel()
        self._pnlTaskEffort: RAMSTKPanel = ValidationTaskEffortPanel()
        # self._pnlProgramEffort: RAMSTKPanel = ProgramEffortPanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_set_cursor_active, "succeed_calculate_validation_task")
        pub.subscribe(
            super().do_set_cursor_active_on_fail, "fail_calculate_validation_task"
        )

        pub.subscribe(self._do_set_record_id, "selected_validation")

    def _do_request_calculate(self, __button: Gtk.ToolButton) -> None:
        """Request to calculate the selected validation task.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        :rtype: None
        """
        super().do_set_cursor_busy()
        pub.sendMessage(
            "request_calculate_validation_task",
            node_id=self.dic_pkeys["record_id"],
        )

    def _do_request_calculate_all(self, __button: Gtk.ToolButton) -> None:
        """Request to calculate program cost and time.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        :rtype: None
        """
        super().do_set_cursor_busy()
        pub.sendMessage(
            "request_calculate_validation_tasks",
        )

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the Verification task record ID.

        :param attributes: the attributes dict for the selected Validation
            task.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["record_id"] = attributes["validation_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the Validation General Data tab.

        :return: None
        :rtype: None
        """
        _hpaned, _vpaned_right = super().do_make_layout_lrr()

        self._pnlTaskDescription.fmt = self.fmt
        self._pnlTaskDescription.do_load_measurement_units(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_MEASUREMENT_UNITS
        )
        self._pnlTaskDescription.do_load_validation_types(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_VALIDATION_TYPE
        )
        _hpaned.pack1(self._pnlTaskDescription, True, True)

        self._pnlTaskEffort.fmt = self.fmt
        self._pnlTaskEffort.do_load_validation_types(
            self.RAMSTK_USER_CONFIGURATION.RAMSTK_VALIDATION_TYPE
        )
        # self._pnlProgramEffort.fmt = self.fmt
        _vpaned_right.pack1(self._pnlTaskEffort, True, True)
        # _vpaned_right.pack2(self._pnlProgramEffort, True, True)

        self.show_all()



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.allocation.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Allocation Package Record Model."""

# Third Party Imports
# noinspection PyPackageRequirements
from sqlalchemy import Column, Float, ForeignKey, Integer

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKAllocationRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_allocation table in RAMSTK Program database.

    This table shares a One-to-One relationship with ramstk_hardware.
    """

    __defaults__ = {
        "availability_alloc": 0.0,
        "duty_cycle": 100.0,
        "env_factor": 1,
        "goal_measure_id": 1,
        "hazard_rate_alloc": 0.0,
        "hazard_rate_goal": 0.0,
        "included": 1,
        "int_factor": 1,
        "allocation_method_id": 1,
        "mission_time": 100.0,
        "mtbf_alloc": 0.0,
        "mtbf_goal": 0.0,
        "n_sub_systems": 1,
        "n_sub_elements": 1,
        "parent_id": 1,
        "percent_weight_factor": 0.0,
        "reliability_alloc": 1.0,
        "reliability_goal": 1.0,
        "op_time_factor": 1,
        "soa_factor": 1,
        "weight_factor": 1.0,
    }
    __tablename__ = "ramstk_allocation"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    hardware_id = Column(
        "fld_hardware_id",
        Integer,
        ForeignKey("ramstk_hardware.fld_hardware_id", ondelete="CASCADE"),
        primary_key=True,
        nullable=False,
    )

    availability_alloc = Column(
        "fld_availability_alloc", Float, default=__defaults__["availability_alloc"]
    )
    duty_cycle = Column("fld_duty_cycle", Float, default=__defaults__["duty_cycle"])
    env_factor = Column("fld_env_factor", Integer, default=__defaults__["env_factor"])
    goal_measure_id = Column(
        "fld_goal_measure_id", Integer, default=__defaults__["goal_measure_id"]
    )
    hazard_rate_alloc = Column(
        "fld_hazard_rate_alloc", Float, default=__defaults__["hazard_rate_alloc"]
    )
    hazard_rate_goal = Column(
        "fld_hazard_rate_goal", Float, default=__defaults__["hazard_rate_goal"]
    )
    included = Column("fld_included", Integer, default=__defaults__["included"])
    int_factor = Column("fld_int_factor", Integer, default=__defaults__["int_factor"])
    allocation_method_id = Column(
        "fld_allocation_method_id",
        Integer,
        default=__defaults__["allocation_method_id"],
    )
    mission_time = Column(
        "fld_mission_time", Float, default=__defaults__["mission_time"]
    )
    mtbf_alloc = Column("fld_mtbf_alloc", Float, default=__defaults__["mtbf_alloc"])
    mtbf_goal = Column("fld_mtbf_goal", Float, default=__defaults__["mtbf_goal"])
    n_sub_systems = Column(
        "fld_n_sub_systems", Integer, default=__defaults__["n_sub_systems"]
    )
    n_sub_elements = Column(
        "fld_n_sub_elements", Integer, default=__defaults__["n_sub_elements"]
    )
    parent_id = Column("fld_parent_id", Integer, default=__defaults__["parent_id"])
    percent_weight_factor = Column(
        "fld_percent_weight_factor",
        Float,
        default=__defaults__["percent_weight_factor"],
    )
    reliability_alloc = Column(
        "fld_reliability_alloc", Float, default=__defaults__["reliability_alloc"]
    )
    reliability_goal = Column(
        "fld_reliability_goal", Float, default=__defaults__["reliability_goal"]
    )
    op_time_factor = Column(
        "fld_op_time_factor", Integer, default=__defaults__["op_time_factor"]
    )
    soa_factor = Column("fld_soa_factor", Integer, default=__defaults__["soa_factor"])
    weight_factor = Column(
        "fld_weight_factor", Integer, default=__defaults__["weight_factor"]
    )

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self):
        """Retrieve current values of RAMSTKAllocation data model attributes.

        :return: {revision_id, hardware_id, availability_alloc, env_factor,
                  goal_measure_id, hazard_rate_alloc, hazard_rate_goal,
                  included, int_factor, method_id, mtbf_alloc, mtbf_goal,
                  n_sub_systems, n_sub_elements, parent_id, percent_wt_factor,
                  reliability_alloc, reliability_goal, op_time_factor,
                  soa_factor, weight_factor} pairs.
        :rtype: dict
        """
        _attributes = {
            "revision_id": self.revision_id,
            "hardware_id": self.hardware_id,
            "availability_alloc": self.availability_alloc,
            "duty_cycle": self.duty_cycle,
            "env_factor": self.env_factor,
            "goal_measure_id": self.goal_measure_id,
            "hazard_rate_alloc": self.hazard_rate_alloc,
            "hazard_rate_goal": self.hazard_rate_goal,
            "included": self.included,
            "int_factor": self.int_factor,
            "allocation_method_id": self.allocation_method_id,
            "mission_time": self.mission_time,
            "mtbf_alloc": self.mtbf_alloc,
            "mtbf_goal": self.mtbf_goal,
            "n_sub_systems": self.n_sub_systems,
            "n_sub_elements": self.n_sub_elements,
            "parent_id": self.parent_id,
            "percent_weight_factor": self.percent_weight_factor,
            "reliability_alloc": self.reliability_alloc,
            "reliability_goal": self.reliability_goal,
            "op_time_factor": self.op_time_factor,
            "soa_factor": self.soa_factor,
            "weight_factor": self.weight_factor,
        }

        return _attributes



# Standard Library Imports
from typing import Any, Dict, Tuple

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.views.gtk3 import GdkPixbuf as GdkPixbuf
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKCheckButton as RAMSTKCheckButton
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel
from ramstk.views.gtk3.widgets import RAMSTKTextView as RAMSTKTextView
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel

class HardwareTreePanel(RAMSTKTreePanel):
    _select_msg: str
    _tag: str
    _title: Any
    _dic_row_loader: Any
    dic_attribute_widget_map: Any
    dic_icons: Any
    def __init__(self) -> None: ...
    def _on_module_switch(self, module: str = ...) -> None: ...
    _record_id: Any
    _parent_id: Any
    def _on_row_change(self, selection: Gtk.TreeSelection) -> None: ...
    def __do_load_hardware(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter: ...

class HardwareGeneralDataPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    chkRepairable: RAMSTKComboBox
    cmbCategory: RAMSTKComboBox
    cmbSubcategory: RAMSTKComboBox
    txtAltPartNum: RAMSTKEntry
    txtCompRefDes: RAMSTKEntry
    txtDescription: RAMSTKTextView
    txtFigureNumber: RAMSTKEntry
    txtLCN: RAMSTKEntry
    txtName: RAMSTKEntry
    txtPageNumber: RAMSTKEntry
    txtPartNumber: RAMSTKEntry
    txtRefDes: RAMSTKEntry
    txtSpecification: RAMSTKEntry
    dic_attribute_widget_map: Any
    dicSubcategories: Any
    def __init__(self) -> None: ...
    def do_load_categories(self, category: Dict[int, str]) -> None: ...
    def _do_load_subcategories(self, category_id: int) -> None: ...
    def _do_set_comp_ref_des(self, comp_ref_des: str) -> None: ...
    def _request_load_component(self, combo: RAMSTKComboBox) -> None: ...
    def _request_load_subcategories(self, combo: RAMSTKComboBox) -> None: ...

class HardwareLogisticsPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    cmbCostType: RAMSTKComboBox
    cmbManufacturer: RAMSTKComboBox
    txtCAGECode: RAMSTKEntry
    txtCost: RAMSTKEntry
    txtNSN: RAMSTKEntry
    txtQuantity: RAMSTKEntry
    txtYearMade: RAMSTKEntry
    dic_attribute_widget_map: Any
    def __init__(self) -> None: ...
    def do_load_cost_types(self) -> None: ...
    def do_load_manufacturers(
        self, manufacturers: Dict[int, Tuple[str, str, str]]
    ) -> None: ...
    def _do_load_cage_code(self, combo: RAMSTKComboBox) -> None: ...

class HardwareMiscellaneousPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    chkTagged: RAMSTKCheckButton
    txtAttachments: RAMSTKTextView
    txtRemarks: RAMSTKTextView
    dic_attribute_widget_map: Any
    def __init__(self) -> None: ...



# RAMSTK Package Imports
from ramstk.views.gtk3 import GObject as GObject
from ramstk.views.gtk3 import Gtk as Gtk

class RAMSTKScrolledWindow(Gtk.ScrolledWindow):
    def __init__(self, child: object) -> None: ...



# pylint: disable=unused-import, wrong-import-position
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.__init__.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK GTK3 GUI package."""

# Standard Library Imports
import gettext
import sys

try:
    import gi  # isort:skip

    gi.require_version("Gdk", "3.0")
    gi.require_version("Gtk", "3.0")
except ImportError:
    print("Failed to import package gi; exiting.")
    sys.exit(1)
from gi.repository import Gdk, GdkPixbuf, GObject, Gtk, Pango  # isort:skip

_ = gettext.gettext  # isort:skip

from .desktop import RAMSTKDesktop  # isort:skip



# -*- coding: utf-8 -*-
#
#       ramstk.models.environment.table.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Environment Table Model."""

# Standard Library Imports
from typing import Any, Dict, Type

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseTable, RAMSTKEnvironmentRecord


class RAMSTKEnvironmentTable(RAMSTKBaseTable):
    """Contain the attributes and methods of the Environment data manager."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _db_id_colname = "fld_environment_id"
    _db_tablename = "ramstk_environment"
    _select_msg = "selected_revision"
    _tag = "environment"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize an Environment data manager instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_id_columns = [
            "revision_id",
            "mission_id",
            "phase_id",
            "environment_id",
            "parent_id",
            "record_id",
        ]

        # Initialize private scalar attributes.
        self._record: Type[RAMSTKEnvironmentRecord] = RAMSTKEnvironmentRecord

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pkey = "environment_id"

        # Subscribe to PyPubSub messages.

    def do_get_new_record(  # pylint: disable=method-hidden
        self, attributes: Dict[str, Any]
    ) -> object:
        """Gets a new record instance with attributes set.

        :param attributes: the dict of attribute values to assign to the new record.
        :return: None
        :rtype: None
        """
        _new_record = self._record()
        _new_record.revision_id = attributes["revision_id"]
        _new_record.mission_id = attributes["mission_id"]
        _new_record.phase_id = attributes["phase_id"]
        _new_record.environment_id = self.last_id + 1

        return _new_record



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.utilities import string_to_boolean as string_to_boolean
from ramstk.views.gtk3 import Gdk as Gdk
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKComboBox as RAMSTKComboBox
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKFileChooserButton as RAMSTKFileChooserButton
from ramstk.views.gtk3.widgets import RAMSTKFixedPanel as RAMSTKFixedPanel
from ramstk.views.gtk3.widgets import RAMSTKLabel as RAMSTKLabel
from ramstk.views.gtk3.widgets import RAMSTKTreePanel as RAMSTKTreePanel
from ramstk.views.gtk3.widgets import RAMSTKTreeView as RAMSTKTreeView

class GeneralPreferencesPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    btnConfDir: RAMSTKFileChooserButton
    btnDataDir: RAMSTKFileChooserButton
    btnIconDir: RAMSTKFileChooserButton
    btnLogDir: RAMSTKFileChooserButton
    cmbModuleBookTabPosition: RAMSTKComboBox
    cmbWorkBookTabPosition: RAMSTKComboBox
    cmbListBookTabPosition: RAMSTKComboBox
    cmbReportSize: RAMSTKComboBox
    txtFRMultiplier: RAMSTKEntry
    txtDecimalPlaces: RAMSTKEntry
    txtMissionTime: RAMSTKEntry
    _configuration: RAMSTKUserConfiguration
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _do_load_panel(self, configuration: RAMSTKUserConfiguration) -> None: ...
    def _do_load_comboboxes(self) -> None: ...
    def _do_select_path(self, button: Gtk.FileChooserButton, index: int) -> None: ...
    def _on_changed_combo(self, combo: RAMSTKComboBox, index: int) -> None: ...

class LookFeelPreferencesPanel(RAMSTKFixedPanel):
    _select_msg: str
    _tag: str
    _title: str
    btnRevisionBGColor: Gtk.ColorButton
    btnRevisionFGColor: Gtk.ColorButton
    btnFunctionBGColor: Gtk.ColorButton
    btnFunctionFGColor: Gtk.ColorButton
    btnRequirementsBGColor: Gtk.ColorButton
    btnRequirementsFGColor: Gtk.ColorButton
    btnHardwareBGColor: Gtk.ColorButton
    btnHardwareFGColor: Gtk.ColorButton
    btnValidationBGColor: Gtk.ColorButton
    btnValidationFGColor: Gtk.ColorButton
    _configuration: RAMSTKUserConfiguration
    dic_attribute_widget_map: Dict[str, List[Any]]
    def __init__(self) -> None: ...
    def _do_load_panel(self, configuration: RAMSTKUserConfiguration) -> None: ...
    def _do_set_color(
        self, colorbutton: Gtk.ColorButton, ramstk_color: int
    ) -> None: ...
    def _do_set_callbacks(self) -> None: ...
    def _do_set_properties(self) -> None: ...

class TreeLayoutPreferencesPanel(RAMSTKTreePanel):
    _select_msg: Any
    _tag: str
    _title: str
    cmbFormatFiles: RAMSTKComboBox
    tvwTreeView: RAMSTKTreeView
    _lst_labels: List[str]
    _configuration: RAMSTKUserConfiguration
    fmt_file: str
    def __init__(self) -> None: ...
    def _do_load_comboboxes(self) -> None: ...
    def _do_load_format(self, module: str) -> None: ...
    def _do_load_panel(self, configuration: RAMSTKUserConfiguration) -> None: ...
    def _on_combo_changed(self, combo: RAMSTKComboBox) -> None: ...
    def __do_make_treeview(self) -> None: ...
    def __make_ui(self) -> None: ...



# pylint: disable=non-parent-init-called
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.widgets.dialog.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTK Dialog Module."""

# Standard Library Imports
import os
from datetime import datetime
from typing import Any, Dict, List, Optional, Tuple

# RAMSTK Package Imports
from ramstk.db.base import BaseDatabase
from ramstk.views.gtk3 import GdkPixbuf, GObject, Gtk, Pango, _
from ramstk.views.gtk3.widgets.button import RAMSTKButton, RAMSTKCheckButton
from ramstk.views.gtk3.widgets.combo import RAMSTKComboBox
from ramstk.views.gtk3.widgets.entry import RAMSTKEntry
from ramstk.views.gtk3.widgets.label import do_make_label_group


class RAMSTKDialog(Gtk.Dialog):
    """The RAMSTK Dialog class."""

    def __init__(self, dlgtitle: str, **kwargs: Any) -> None:
        """Initialize a RAMSTK Dialog widget.

        Accepts the following keyword arguments:
            * *dlgparent* (tuple) -- the parent window to associate the
                Gtk.Dialog() with.
            * *dlgflags* (tuple) -- the flags that control the operation of the
                Gtk.Dialog().  Default is Gtk.DialogFlags.MODAL and
                Gtk.DialogFlags.DESTROY_WITH_PARENT.
            * *dlgbuttons* (tuple) -- the buttons to display and their response
                values.  Default is Gtk.STOCK_OK <==> Gtk.ResponseType.ACCEPT
                Gtk.STOCK_CANCEL <==> Gtk.ResponseType.CANCEL

        :param dlgtitle: the title text for the Gtk.Dialog().
        """
        _dlgbuttons = kwargs.get(
            "dlgbuttons",
            (
                Gtk.STOCK_OK,
                Gtk.ResponseType.OK,
                Gtk.STOCK_CANCEL,
                Gtk.ResponseType.CANCEL,
            ),
        )
        _dlgparent = kwargs.get("dlgparent", None)

        super().__init__()

        self.set_title(dlgtitle)
        self.set_transient_for(_dlgparent)
        self.add_buttons(*_dlgbuttons)

        self.set_destroy_with_parent(True)
        self.set_modal(True)

    def do_destroy(self) -> None:  # pylint: disable=arguments-differ
        """Destroy the RAMSTK Dialog."""
        self.destroy()

    def do_run(self) -> Any:
        """Run the RAMSTK Dialog."""
        return self.run()


class RAMSTKDatabaseSelect(RAMSTKDialog):
    """The RAMSTK Database Selection Dialog."""

    def __init__(self, dlgtitle: str, **kwargs: Any) -> None:
        """Initialize an instance of the RAMSTKdatabaseSelect class."""
        super().__init__(dlgtitle, **kwargs)

        # Initialize private dict attributes.
        self._dao: BaseDatabase = kwargs.get("dao", BaseDatabase)
        self._dic_icons = kwargs.get("icons", {})

        # Initialize private list attributes.
        self._lst_databases: List[str] = []
        self._lst_labels: List[str] = [
            _("Database Dialect:"),
            _("Database Server:"),
            _("Server Port:"),
            _("Database Name:"),
            _("RAMSTK User:"),
            _("RAMSTK Password:"),
        ]

        # Initialize private scalar attributes.
        self._old_host: str = ""

        # Initialize public scalar attributes.
        self.btnRefresh: RAMSTKButton = RAMSTKButton(label="")
        self.btnSave: RAMSTKCheckButton = RAMSTKCheckButton(
            label="Save Connection Parameters"
        )
        self.cmbDialect: RAMSTKComboBox = RAMSTKComboBox()
        self.txtHost: RAMSTKEntry = RAMSTKEntry()
        self.txtPort: RAMSTKEntry = RAMSTKEntry()
        self.txtDatabase: RAMSTKEntry = RAMSTKEntry()
        self.txtUser: RAMSTKEntry = RAMSTKEntry()
        self.txtPassword: RAMSTKEntry = RAMSTKEntry()
        self.tvwTreeView: Gtk.TreeView = Gtk.TreeView()

        self._lst_widgets = [
            self.cmbDialect,
            self.txtHost,
            self.txtPort,
            self.txtDatabase,
            self.txtUser,
            self.txtPassword,
        ]

        # Initialize public dict attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.database: Dict[str, str] = kwargs.get("database", {})
        self.exists: bool = False

        self.__do_set_properties()
        self.__make_ui()
        self.__do_load_combobox()
        self.__do_set_callbacks()

        self.__do_load_databases()

    def do_run(self) -> Gtk.ResponseType:
        """Run the RAMSTKFileChooser dialog.

        :return: _return
        :rtype: Gtk.ResponseType
        """
        _return = Gtk.ResponseType.CANCEL

        if self.run() == Gtk.ResponseType.OK:
            self._get_database()
            self.exists = self.database["database"] in self._lst_databases
            _return = Gtk.ResponseType.OK
        elif self.run() == Gtk.ResponseType.CANCEL:
            _return = Gtk.ResponseType.CANCEL

        return _return

    def _get_database(self):
        """Get the name of the selected database.

        :return: the database connection parameters.
        :rtype: dict
        """
        self.database["dialect"] = self.cmbDialect.get_value()
        self.database["host"] = self.txtHost.do_get_text()
        self.database["port"] = self.txtPort.do_get_text()
        self.database["database"] = self.txtDatabase.do_get_text()
        self.database["user"] = self.txtUser.do_get_text()
        self.database["password"] = self.txtPassword.do_get_text()

        # IF the host was changed, reload the database list, otherwise keep
        # going.
        if self.database["host"] != self._old_host:
            self.__do_load_databases()

    def _request_load_databases(self, __button: RAMSTKButton) -> None:
        """Re-load the database list.

        :param __button: the RAMSTKButton() that called this method on
            'clicked'.
        :return: None
        :rtype: None
        """
        self.database["dialect"] = self.cmbDialect.get_value()
        self.database["host"] = self.txtHost.do_get_text()
        self.database["port"] = self.txtPort.do_get_text()
        self.database["database"] = self.txtDatabase.do_get_text()
        self.database["user"] = self.txtUser.do_get_text()
        self.database["password"] = self.txtPassword.do_get_text()

        self.__do_load_databases()

    def __do_load_combobox(self) -> None:
        """Load the dialect RAMSTKComboBox."""
        self.cmbDialect.do_load_combo([["postgres", "", ""]])

    def __do_load_databases(self) -> None:
        """Read the database server and load the database list.

        :return: None
        :rtype: None
        """
        _dialect = 0

        self._old_host = self.database["host"]

        self.txtHost.do_update(self.database["host"])
        self.txtPort.do_update(self.database["port"])
        self.txtDatabase.do_update(self.database["database"])
        self.txtUser.do_update(self.database["user"])
        self.txtPassword.do_update(self.database["password"])

        _model = self.tvwTreeView.get_model()
        _model.clear()

        if (
            self.database["dialect"] == "postgres"
            and self.database["user"] != "first_run"
        ):
            _dialect = 1
            _stored_db = self.database["database"]
            self.database["database"] = "postgres"

            for _db in self._dao.get_database_list(self.database):
                _model.append([_db])
                self._lst_databases.append(_db)

            self.database["database"] = _stored_db

        self.cmbDialect.do_update(_dialect)

    def __do_set_callbacks(self) -> None:
        """Set widget callback methods."""
        self.btnRefresh.connect("clicked", self._request_load_databases)
        self.tvwTreeView.selection.connect("changed", self.__on_row_change)

    def __do_set_properties(self) -> None:
        """Set the properties of the widgets."""
        _image = Gtk.Image()
        _icon = GdkPixbuf.Pixbuf.new_from_file_at_size(
            self._dic_icons["refresh"], 22, 22
        )
        _image.set_from_pixbuf(_icon)
        self.btnRefresh.set_image(_image)
        self.btnRefresh.do_set_properties(
            tooltip=_("Refresh server database list."), width=50
        )
        self.btnSave.do_set_properties(
            tooltip=_("Save connection information to configuration file.")
        )
        self.cmbDialect.do_set_properties(
            tooltip=_("Select SQL server dialect for this connection."), width=300
        )
        self.txtHost.do_set_properties(
            tooltip=_("Enter the database server hostname."), width=300
        )
        self.txtPort.do_set_properties(
            tooltip=_("Enter the port number the database server listens on."),
            width=300,
        )
        self.txtDatabase.do_set_properties(
            tooltip=_("Enter the name of the database to connect."), width=300
        )
        self.txtUser.do_set_properties(
            tooltip=_("Enter the user name for the database server."), width=300
        )
        self.txtPassword.do_set_properties(
            tooltip=_("Enter the user password for the database server."), width=300
        )

        self.tvwTreeView.selection = self.tvwTreeView.get_selection()

    def __make_ui(self) -> None:
        """Build the GUI."""
        self.set_modal(True)

        _fixed = Gtk.Fixed()

        (_x_pos, _labels) = do_make_label_group(self._lst_labels)
        _y_pos = 5
        for _idx, _label in enumerate(_labels):
            _fixed.put(_label, 5, _y_pos)
            _fixed.put(self._lst_widgets[_idx], _x_pos + 10, _y_pos)
            _y_pos += 35

        # Add the refresh and save buttons.
        _fixed.put(self.btnRefresh, _x_pos + 315, 40)
        _fixed.put(self.btnSave, _x_pos + 315, 180)
        self.vbox.pack_start(_fixed, True, True, 10)

        _model = Gtk.ListStore(GObject.TYPE_STRING)
        self.tvwTreeView.set_model(_model)

        _cell = Gtk.CellRendererText()
        _cell.set_alignment(0.1, 0.5)
        _cell.set_property("background", "light gray")
        _cell.set_property("editable", False)
        _cell.set_property("foreground", "#000000")
        _cell.set_property("wrap-width", 250)
        _cell.set_property("wrap-mode", Pango.WrapMode.WORD_CHAR)

        _column = Gtk.TreeViewColumn("Available Databases")
        _column.pack_start(_cell, True)
        _column.set_attributes(_cell, text=0)

        self.tvwTreeView.append_column(_column)

        _scrollwindow = Gtk.ScrolledWindow()
        _scrollwindow.set_min_content_height(200)
        _scrollwindow.set_min_content_width(500)
        _scrollwindow.add(self.tvwTreeView)

        self.vbox.pack_end(_scrollwindow, True, True, 0)
        self.vbox.show_all()

    def __on_row_change(self, selection: Gtk.TreeSelection) -> None:
        """Respond to changes in database list selection."""
        _model, _row = selection.get_selected()
        self.txtDatabase.do_update(_model.get_value(_row, 0))


class RAMSTKDateSelect(Gtk.Dialog):
    """The RAMSTK Date Selection Dialog."""

    def __init__(self, **kwargs) -> None:
        """Initialize an instance of the RAMSTKDateSelect class."""
        _dlgparent = kwargs.get("dlgparent", None)

        super().__init__(self)

        self.set_transient_for(_dlgparent)
        self.set_destroy_with_parent(True)
        self.set_modal(True)

        self.add_buttons(Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT)
        self.set_title(_("Select Date"))

        self._calendar = Gtk.Calendar()
        self.vbox.pack_start(self._calendar, True, True, 0)
        self.vbox.show_all()

    def do_destroy(self) -> None:  # pylint: disable=arguments-differ
        """Destroy the RAMSTKDateSelect dialog."""
        self.destroy()

    def do_run(self) -> Any:
        """Run the RAMSTKDateSelect dialog.

        :return: the selected date or the default date if the dialog is
            cancelled.
        :rtype: str
        """
        if self.run() == Gtk.ResponseType.ACCEPT:
            _date = self._calendar.get_date()
            _date = (
                datetime(
                    _date[0],
                    _date[1] + 1,
                    _date[2],
                )
                .date()
                .strftime("%Y-%m-%d")
            )
        else:
            _date = "1970-01-01"

        return _date


class RAMSTKFileChooser(Gtk.FileChooserDialog):
    """The RAMSTK File Chooser Dialog class."""

    def __init__(self, title: str, parent: object) -> None:
        """Initialize an instance of the RAMSTKFileChooser dialog.

        :param title: the title of the dialog.
        :param object parent: the parent window for the dialog.
        """
        Gtk.FileChooserDialog.__init__(self)

        self.add_buttons(
            Gtk.STOCK_OK,
            Gtk.ResponseType.ACCEPT,
            Gtk.STOCK_CANCEL,
            Gtk.ResponseType.REJECT,
        )

        self.set_title(title)
        self.set_transient_for(parent)
        self.set_destroy_with_parent(True)
        self.set_modal(True)

        self.set_action(Gtk.FileChooserAction.SAVE)

        _filter = Gtk.FileFilter()
        _filter.set_name(_("Excel Files"))
        _filter.add_pattern("*.xls")
        _filter.add_pattern("*xlsm")
        _filter.add_pattern("*xlsx")
        self.add_filter(_filter)
        _filter = Gtk.FileFilter()
        _filter.set_name(_("Delimited Text Files"))
        _filter.add_pattern("*.csv")
        _filter.add_pattern("*.txt")
        self.add_filter(_filter)
        _filter = Gtk.FileFilter()
        _filter.set_name("All files")
        _filter.add_pattern("*")
        self.add_filter(_filter)

    def do_destroy(self) -> None:  # pylint: disable=arguments-differ
        """Destroy the RAMSTKFileChooser dialog."""
        self.destroy()

    def do_run(self) -> Tuple[Optional[Any], Optional[Any]]:
        """Run the RAMSTKFileChooser dialog.

        :return: (_filename, _extension); the file name and file extension of
                 the selected file.
        :rtype: (str, str) or (None, None)
        """
        _filename = None
        _extension = None

        if self.run() == Gtk.ResponseType.ACCEPT:
            _filename = self.get_filename()
            # pylint: disable=unused-variable
            __, _extension = os.path.splitext(_filename)
        elif self.run() == Gtk.ResponseType.REJECT:
            self.do_destroy()

        return _filename, _extension


class RAMSTKMessageDialog(Gtk.MessageDialog):
    """The RAMSTK Message Dialog class.

    It used for RAMSTK error, warning, and information messages.
    """

    def __init__(self, parent: Gtk.Window = None) -> None:
        """Initialize runtime error, warning, and information dialogs.

        :param parent: the parent Gtk.Window(), if any, for the dialog.
        """
        Gtk.MessageDialog.__init__(self)

        self.set_transient_for(parent)
        self.set_destroy_with_parent(True)
        self.set_modal(True)

        self.show_all()

    def do_set_message(self, message: str) -> None:
        """Set the message to display in the dialog.

        :param message: the message to display.
        :return: None
        :rtype: None
        """
        self.set_markup(message)

    def do_set_message_type(self, message_type: str = "error") -> None:
        """Set RAMSTKMessageDialog message type.

        :param message_type: the RAMSTKMessageDialog message type.
            Options are error, warning, information, or question.  Default
            is error.
        :return: None
        :rtype: None
        """
        _message_type = Gtk.MessageType.INFO

        _prompt = self.get_property("text")
        # Set the prompt to bold text with a hyperlink to the RAMSTK bugs
        # e-mail address.
        _hyper = (
            "<a href='mailto:bugs@reliaqual.com?subject=RAMSTK BUG "
            "REPORT: <ADD SHORT PROBLEM DESCRIPTION>&amp;"
            "body=RAMSTK MODULE:%0d%0a%0d%0a"
            "RAMSTK VERSION:%20%0d%0a%0d%0a"
            "YOUR HARDWARE:%20%0d%0a%0d%0a"
            "YOUR OS:%20%0d%0a%0d%0a"
            "DETAILED PROBLEM DESCRIPTION:%20%0d%0a'>"
        )
        _prompt = (
            "<b>"
            + _prompt
            + _(
                "  Check the error log for additional information "
                "(if any).  Please e-mail <span foreground='blue' "
                "underline='single'>",
            )
            + _hyper
            + _(
                "bugs@reliaqual.com</a></span> with a detailed "
                "description of the problem, the workflow you are "
                "using and the error log attached if the problem "
                "persists.</b>",
            )
        )
        if message_type == "error":
            self.set_markup(_prompt)
            _message_type = Gtk.MessageType.ERROR
            self.add_buttons("_OK", Gtk.ResponseType.OK)
        elif message_type == "warning":
            self.set_markup(_prompt)
            _message_type = Gtk.MessageType.WARNING
            self.add_buttons("_OK", Gtk.ResponseType.OK)
        elif message_type == "information":
            _message_type = Gtk.MessageType.INFO
            self.add_buttons("_OK", Gtk.ResponseType.OK)
        elif message_type == "question":
            _message_type = Gtk.MessageType.QUESTION
            self.add_buttons("_Yes", Gtk.ResponseType.YES, "_No", Gtk.ResponseType.NO)

        self.set_property("message-type", _message_type)

    def do_run(self) -> Any:
        """Run the RAMSTK Message Dialog."""
        return self.run()

    def do_destroy(self) -> None:  # pylint: disable=arguments-differ
        """Destroy the RAMSTK Message Dialog."""
        self.destroy()



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKRPN.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKRPN Table Module."""

# Third Party Imports
from sqlalchemy import Column, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKRPN(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent table ramstk_rpn in the RAMSTK Common database."""

    __defaults__ = {
        "name": "RPN Name",
        "description": "RPN Description",
        "rpn_type": "",
        "value": 0,
    }
    __tablename__ = "ramstk_rpn"
    __table_args__ = {"extend_existing": True}

    rpn_id = Column(
        "fld_rpn_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    name = Column("fld_name", String(512), default=__defaults__["name"])
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    rpn_type = Column("fld_rpn_type", String(256), default=__defaults__["rpn_type"])
    value = Column("fld_value", Integer, default=__defaults__["value"])

    def get_attributes(self):
        """Retrieve the current values of the RAMSTKRPN data model attributes.

        :return: {}rpn_id, name, description, rpn_type, value} key:value pairs
        :rtype: dict
        """
        _values = {
            "rpn_id": self.rpn_id,
            "name": self.name,
            "description": self.description,
            "rpn_type": self.rpn_type,
            "value": self.value,
        }

        return _values



# Standard Library Imports
from typing import Any, Dict

# RAMSTK Local Imports
from .models import capacitor as capacitor
from .models import connection as connection
from .models import crystal as crystal
from .models import efilter as efilter
from .models import fuse as fuse
from .models import inductor as inductor
from .models import integratedcircuit as integratedcircuit
from .models import lamp as lamp
from .models import meter as meter
from .models import relay as relay
from .models import resistor as resistor
from .models import semiconductor as semiconductor
from .models import switch as switch

def do_predict_active_hazard_rate(**attributes: Dict[str, Any]) -> float: ...
def _do_calculate_part_count(**attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def _do_calculate_part_stress(**attributes: Dict[str, Any]) -> Dict[str, Any]: ...
def _get_environment_factor(
    category_id: int,
    environment_active_id: int,
    subcategory_id: int = ...,
    quality_id: int = ...,
) -> float: ...
def _get_part_count_quality_factor(
    category_id: int, subcategory_id: int, quality_id: int
) -> float: ...
def _get_part_stress_quality_factor(
    category_id: int, subcategory_id: int, quality_id: int
) -> float: ...



# Standard Library Imports
from typing import Callable, List

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.utilities import integer_to_boolean as integer_to_boolean
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKBaseView as RAMSTKBaseView
from ramstk.views.gtk3.widgets import RAMSTKLabel as RAMSTKLabel

# RAMSTK Local Imports
from . import GeneralPreferencesPanel as GeneralPreferencesPanel
from . import LookFeelPreferencesPanel as LookFeelPreferencesPanel
from . import TreeLayoutPreferencesPanel as TreeLayoutPreferencesPanel

class PreferencesDialog(RAMSTKBaseView):
    _tag: str
    _pixbuf: bool
    _tablabel: str
    _tabtooltip: str
    _lst_callbacks: List[Callable]
    _lst_icons: List[str]
    _lst_tooltips: List[str]
    _pnlGeneralPreferences: GeneralPreferencesPanel
    _pnlLookFeel: LookFeelPreferencesPanel
    _pnlTreeViewLayout: TreeLayoutPreferencesPanel
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def _cancel(self, __button: Gtk.Button) -> None: ...
    def _do_request_update(self, __button: Gtk.Button) -> None: ...
    def _do_save_tree_layout(self) -> None: ...
    def __make_ui(self) -> None: ...



# Standard Library Imports
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.db import BaseDatabase

class RAMSTKProgramDB:
    dic_tables: Any
    dic_views: Any
    user_configuration: Any
    program_dao: Any
    def __init__(self) -> None: ...
    def do_create_program(
        self, program_db: BaseDatabase, database: Dict[str, str]
    ) -> None: ...
    def do_open_program(
        self, program_db: BaseDatabase, database: Dict[str, str]
    ) -> None: ...
    def do_close_program(self) -> None: ...
    @staticmethod
    def do_save_program() -> None: ...



# pylint: disable=unused-import, wrong-import-position
# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.hardware.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Hardware Package."""

# RAMSTK Local Imports
from .panel import (
    HardwareGeneralDataPanel,
    HardwareLogisticsPanel,
    HardwareMiscellaneousPanel,
    HardwareTreePanel,
)
from .view import (
    HardwareAssessmentInputView,
    HardwareAssessmentResultsView,
    HardwareGeneralDataView,
    HardwareModuleView,
)



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.stakeholder.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Stakeholder Record Model."""

# Third Party Imports
from sqlalchemy import Column, Float, ForeignKey, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKStakeholderRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_stakeholder table in RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_revision.
    """

    __defaults__ = {
        "customer_rank": 1,
        "description": "Stakeholder Input",
        "group": "",
        "improvement": 0.0,
        "overall_weight": 0.0,
        "planned_rank": 1,
        "priority": 1,
        "requirement_id": 0,
        "stakeholder": "",
        "user_float_1": 1.0,
        "user_float_2": 1.0,
        "user_float_3": 1.0,
        "user_float_4": 1.0,
        "user_float_5": 1.0,
    }
    __tablename__ = "ramstk_stakeholder"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    stakeholder_id = Column(
        "fld_stakeholder_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )

    customer_rank = Column(
        "fld_customer_rank", Integer, default=__defaults__["customer_rank"]
    )
    description = Column("fld_description", String, default=__defaults__["description"])
    group = Column("fld_group", String(128), default=__defaults__["group"])
    improvement = Column("fld_improvement", Float, default=__defaults__["improvement"])
    overall_weight = Column(
        "fld_overall_weight", Float, default=__defaults__["overall_weight"]
    )
    planned_rank = Column(
        "fld_planned_rank", Integer, default=__defaults__["planned_rank"]
    )
    priority = Column("fld_priority", Integer, default=__defaults__["priority"])
    requirement_id = Column(
        "fld_requirement_id", Integer, default=__defaults__["requirement_id"]
    )
    stakeholder = Column(
        "fld_stakeholder", String(128), default=__defaults__["stakeholder"]
    )
    user_float_1 = Column(
        "fld_user_float_1", Float, default=__defaults__["user_float_1"]
    )
    user_float_2 = Column(
        "fld_user_float_2", Float, default=__defaults__["user_float_2"]
    )
    user_float_3 = Column(
        "fld_user_float_3", Float, default=__defaults__["user_float_3"]
    )
    user_float_4 = Column(
        "fld_user_float_4", Float, default=__defaults__["user_float_4"]
    )
    user_float_5 = Column(
        "fld_user_float_5", Float, default=__defaults__["user_float_5"]
    )

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self):
        """Retrieve current values of RAMSTKStakeholder data model attributes.

        :return: {revision_id, stakeholder_id, customer_rank, description,
                  group, improvement, overall_weight, planned_rank, priority,
                  requirement_id, stakeholder, user_float_1, user_float_2,
                  user_float_3, user_float_4, user_float_5} pairs.
        :rtype: tuple
        """
        _attributes = {
            "revision_id": self.revision_id,
            "stakeholder_id": self.stakeholder_id,
            "customer_rank": self.customer_rank,
            "description": self.description,
            "group": self.group,
            "improvement": self.improvement,
            "overall_weight": self.overall_weight,
            "planned_rank": self.planned_rank,
            "priority": self.priority,
            "requirement_id": self.requirement_id,
            "stakeholder": self.stakeholder,
            "user_float_1": self.user_float_1,
            "user_float_2": self.user_float_2,
            "user_float_3": self.user_float_3,
            "user_float_4": self.user_float_4,
            "user_float_5": self.user_float_5,
        }

        return _attributes



# Standard Library Imports
from typing import Any, Tuple

def get_hazard_rate(location: float, scale: float, time: float) -> float: ...
def get_mtbf(location: float, scale: float) -> float: ...
def get_survival(location: float, scale: float, time: float) -> float: ...
def do_fit(data: Any, **kwargs: Any) -> Tuple[float, float]: ...



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.requirement.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Requirement Record Model."""

# Standard Library Imports
from datetime import date

# Third Party Imports
from sqlalchemy import Column, Date, ForeignKey, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord
from ramstk.utilities import none_to_default


class RAMSTKRequirementRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_requirement table in RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_revision.
    """

    __defaults__ = {
        "derived": 0,
        "description": "",
        "figure_number": "",
        "owner": 0,
        "page_number": "",
        "parent_id": 0,
        "priority": 0,
        "requirement_code": "",
        "specification": "",
        "requirement_type": 0,
        "validated": 0,
        "validated_date": date.today(),
        "q_clarity_0": 0,
        "q_clarity_1": 0,
        "q_clarity_2": 0,
        "q_clarity_3": 0,
        "q_clarity_4": 0,
        "q_clarity_5": 0,
        "q_clarity_6": 0,
        "q_clarity_7": 0,
        "q_clarity_8": 0,
        "q_complete_0": 0,
        "q_complete_1": 0,
        "q_complete_2": 0,
        "q_complete_3": 0,
        "q_complete_4": 0,
        "q_complete_5": 0,
        "q_complete_6": 0,
        "q_complete_7": 0,
        "q_complete_8": 0,
        "q_complete_9": 0,
        "q_consistent_0": 0,
        "q_consistent_1": 0,
        "q_consistent_2": 0,
        "q_consistent_3": 0,
        "q_consistent_4": 0,
        "q_consistent_5": 0,
        "q_consistent_6": 0,
        "q_consistent_7": 0,
        "q_consistent_8": 0,
        "q_verifiable_0": 0,
        "q_verifiable_1": 0,
        "q_verifiable_2": 0,
        "q_verifiable_3": 0,
        "q_verifiable_4": 0,
        "q_verifiable_5": 0,
    }
    __tablename__ = "ramstk_requirement"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        ForeignKey("ramstk_revision.fld_revision_id", ondelete="CASCADE"),
        nullable=False,
    )
    requirement_id = Column(
        "fld_requirement_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )

    derived = Column("fld_derived", Integer, default=__defaults__["derived"])
    description = Column("fld_description", String, default=__defaults__["description"])
    figure_number = Column(
        "fld_figure_number", String(256), default=__defaults__["figure_number"]
    )
    owner = Column("fld_owner", Integer, default=__defaults__["owner"])
    page_number = Column(
        "fld_page_number", String(256), default=__defaults__["page_number"]
    )
    parent_id = Column("fld_parent_id", Integer, default=__defaults__["parent_id"])
    priority = Column("fld_priority", Integer, default=__defaults__["priority"])
    requirement_code = Column(
        "fld_requirement_code", String(256), default=__defaults__["requirement_code"]
    )
    specification = Column(
        "fld_specification", String(256), default=__defaults__["specification"]
    )
    requirement_type = Column(
        "fld_requirement_type", Integer, default=__defaults__["requirement_type"]
    )
    validated = Column("fld_validated", Integer, default=__defaults__["validated"])
    validated_date = Column(
        "fld_validated_date", Date, default=__defaults__["validated_date"]
    )

    # Clarity of requirement questions.
    q_clarity_0 = Column("fld_clarity_0", Integer, default=__defaults__["q_clarity_0"])
    q_clarity_1 = Column("fld_clarity_1", Integer, default=__defaults__["q_clarity_1"])
    q_clarity_2 = Column("fld_clarity_2", Integer, default=__defaults__["q_clarity_2"])
    q_clarity_3 = Column("fld_clarity_3", Integer, default=__defaults__["q_clarity_3"])
    q_clarity_4 = Column("fld_clarity_4", Integer, default=__defaults__["q_clarity_4"])
    q_clarity_5 = Column("fld_clarity_5", Integer, default=__defaults__["q_clarity_5"])
    q_clarity_6 = Column("fld_clarity_6", Integer, default=__defaults__["q_clarity_6"])
    q_clarity_7 = Column("fld_clarity_7", Integer, default=__defaults__["q_clarity_7"])
    q_clarity_8 = Column("fld_clarity_8", Integer, default=__defaults__["q_clarity_8"])

    # Completeness of requirement questions.
    q_complete_0 = Column(
        "fld_complete_0", Integer, default=__defaults__["q_complete_0"]
    )
    q_complete_1 = Column(
        "fld_complete_1", Integer, default=__defaults__["q_complete_1"]
    )
    q_complete_2 = Column(
        "fld_complete_2", Integer, default=__defaults__["q_complete_2"]
    )
    q_complete_3 = Column(
        "fld_complete_3", Integer, default=__defaults__["q_complete_3"]
    )
    q_complete_4 = Column(
        "fld_complete_4", Integer, default=__defaults__["q_complete_4"]
    )
    q_complete_5 = Column(
        "fld_complete_5", Integer, default=__defaults__["q_complete_5"]
    )
    q_complete_6 = Column(
        "fld_complete_6", Integer, default=__defaults__["q_complete_6"]
    )
    q_complete_7 = Column(
        "fld_complete_7", Integer, default=__defaults__["q_complete_7"]
    )
    q_complete_8 = Column(
        "fld_complete_8", Integer, default=__defaults__["q_complete_8"]
    )
    q_complete_9 = Column(
        "fld_complete_9", Integer, default=__defaults__["q_complete_9"]
    )

    # Consistency of requirement questions.
    q_consistent_0 = Column(
        "fld_consistent_0", Integer, default=__defaults__["q_consistent_0"]
    )
    q_consistent_1 = Column(
        "fld_consistent_1", Integer, default=__defaults__["q_consistent_1"]
    )
    q_consistent_2 = Column(
        "fld_consistent_2", Integer, default=__defaults__["q_consistent_2"]
    )
    q_consistent_3 = Column(
        "fld_consistent_3", Integer, default=__defaults__["q_consistent_3"]
    )
    q_consistent_4 = Column(
        "fld_consistent_4", Integer, default=__defaults__["q_consistent_4"]
    )
    q_consistent_5 = Column(
        "fld_consistent_5", Integer, default=__defaults__["q_consistent_5"]
    )
    q_consistent_6 = Column(
        "fld_consistent_6", Integer, default=__defaults__["q_consistent_6"]
    )
    q_consistent_7 = Column(
        "fld_consistent_7", Integer, default=__defaults__["q_consistent_7"]
    )
    q_consistent_8 = Column(
        "fld_consistent_8", Integer, default=__defaults__["q_consistent_8"]
    )

    # Verifiablity of requirement questions.
    q_verifiable_0 = Column(
        "fld_verifiable_0", Integer, default=__defaults__["q_verifiable_0"]
    )
    q_verifiable_1 = Column(
        "fld_verifiable_1", Integer, default=__defaults__["q_verifiable_1"]
    )
    q_verifiable_2 = Column(
        "fld_verifiable_2", Integer, default=__defaults__["q_verifiable_2"]
    )
    q_verifiable_3 = Column(
        "fld_verifiable_3", Integer, default=__defaults__["q_verifiable_3"]
    )
    q_verifiable_4 = Column(
        "fld_verifiable_4", Integer, default=__defaults__["q_verifiable_4"]
    )
    q_verifiable_5 = Column(
        "fld_verifiable_5", Integer, default=__defaults__["q_verifiable_5"]
    )

    # Define the relationships to other tables in the RAMSTK Program database.

    def get_attributes(self):
        """Retrieve current values of the Requirement data model attributes.

        :return: {revision_id, requirement_id, derived, description,
                  figure_number, owner, page_number, parent_id, priority,
                  requirement_code, specification, requirement_type, validated,
                  validated_date, q_clarity_0, q_clarity_1, q_clarity_2,
                  q_clarity_3, q_clarity_4, q_clarity_5, q_clarity_6,
                  q_clarity_7, q_clarity_8, q_complete_0, q_complete_1,
                  q_complete_2, q_complete_3, q_complete_4, q_complete_5,
                  q_complete_6, q_complete_7, q_complete_8, q_complete_9,
                  q_consistent_0, q_consistent_1, q_consistent_2,
                  q_consistent_3, q_consistent_4, q_consistent_5,
                  q_consistent_6, q_consistent_7, q_consistent_8,
                  q_verifiable_0, q_verifiable_1, q_verifiable_2,
                  q_verifiable_3, q_verifiable_4, q_verifiable_5} pairs.
        :rtype: dict
        """
        _attributes = {
            "revision_id": self.revision_id,
            "requirement_id": self.requirement_id,
            "derived": self.derived,
            "description": self.description,
            "figure_number": self.figure_number,
            "owner": self.owner,
            "page_number": self.page_number,
            "parent_id": self.parent_id,
            "priority": self.priority,
            "requirement_code": self.requirement_code,
            "specification": self.specification,
            "requirement_type": self.requirement_type,
            "validated": self.validated,
            "validated_date": self.validated_date,
            "q_clarity_0": self.q_clarity_0,
            "q_clarity_1": self.q_clarity_1,
            "q_clarity_2": self.q_clarity_2,
            "q_clarity_3": self.q_clarity_3,
            "q_clarity_4": self.q_clarity_4,
            "q_clarity_5": self.q_clarity_5,
            "q_clarity_6": self.q_clarity_6,
            "q_clarity_7": self.q_clarity_7,
            "q_clarity_8": self.q_clarity_8,
            "q_complete_0": self.q_complete_0,
            "q_complete_1": self.q_complete_1,
            "q_complete_2": self.q_complete_2,
            "q_complete_3": self.q_complete_3,
            "q_complete_4": self.q_complete_4,
            "q_complete_5": self.q_complete_5,
            "q_complete_6": self.q_complete_6,
            "q_complete_7": self.q_complete_7,
            "q_complete_8": self.q_complete_8,
            "q_complete_9": self.q_complete_9,
            "q_consistent_0": self.q_consistent_0,
            "q_consistent_1": self.q_consistent_1,
            "q_consistent_2": self.q_consistent_2,
            "q_consistent_3": self.q_consistent_3,
            "q_consistent_4": self.q_consistent_4,
            "q_consistent_5": self.q_consistent_5,
            "q_consistent_6": self.q_consistent_6,
            "q_consistent_7": self.q_consistent_7,
            "q_consistent_8": self.q_consistent_8,
            "q_verifiable_0": self.q_verifiable_0,
            "q_verifiable_1": self.q_verifiable_1,
            "q_verifiable_2": self.q_verifiable_2,
            "q_verifiable_3": self.q_verifiable_3,
            "q_verifiable_4": self.q_verifiable_4,
            "q_verifiable_5": self.q_verifiable_5,
        }

        return _attributes

    def create_code(self, prefix: str) -> None:
        """Create Requirement code based on the requirement type and it's ID.

        :param prefix: the prefix to use for the Requirement code.
        :return: None
        :rtype: None
        """
        # Pad the suffix (Requirement ID) with zeros so the suffix is four
        # characters wide and then create the code.
        _zeds = 4 - len(str(self.requirement_id))
        _pad = "0" * _zeds
        _code = "{0:s}-{1:s}{2:d}".format(prefix, _pad, self.requirement_id)

        self.requirement_code = str(none_to_default(_code, ""))



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.control.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Control Record Model."""

# Third Party Imports
from sqlalchemy import Column, ForeignKeyConstraint, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKControlRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent table ramstk_control in the RAMSTK Program database.

    This table shares a Many-to-One relationship with ramstk_cause.
    """

    __defaults__ = {"description": "", "type_id": ""}
    __tablename__ = "ramstk_control"
    __table_args__ = (
        ForeignKeyConstraint(
            [
                "fld_revision_id",
                "fld_hardware_id",
                "fld_mode_id",
                "fld_mechanism_id",
                "fld_cause_id",
            ],
            [
                "ramstk_cause.fld_revision_id",
                "ramstk_cause.fld_hardware_id",
                "ramstk_cause.fld_mode_id",
                "ramstk_cause.fld_mechanism_id",
                "ramstk_cause.fld_cause_id",
            ],
        ),
        {"extend_existing": True},
    )

    revision_id = Column("fld_revision_id", Integer, primary_key=True, nullable=False)
    hardware_id = Column(
        "fld_hardware_id", Integer, primary_key=True, default=-1, nullable=False
    )
    mode_id = Column("fld_mode_id", Integer, primary_key=True, nullable=False)
    mechanism_id = Column("fld_mechanism_id", Integer, primary_key=True, nullable=False)
    cause_id = Column(
        "fld_cause_id", Integer, primary_key=True, nullable=False, unique=True
    )
    control_id = Column(
        "fld_control_id", Integer, primary_key=True, autoincrement=True, nullable=False
    )

    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )
    type_id = Column("fld_type_id", String(512), default=__defaults__["type_id"])

    # Define the relationships to other tables in the RAMSTK Program database.
    cause = relationship("RAMSTKCauseRecord", back_populates="control")  # type: ignore

    is_mode = False
    is_mechanism = False
    is_cause = False
    is_control = True
    is_action = False

    def get_attributes(self):
        """Retrieve current values of the RAMSTKControl data model attributes.

        :return: {cause_id, control_id, description, type_id} pairs.
        :rtype: dict
        """
        _attributes = {
            "cause_id": self.cause_id,
            "control_id": self.control_id,
            "description": self.description,
            "type_id": self.type_id,
        }

        return _attributes



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord

class RAMSTKAllocationRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    revision_id: Any
    hardware_id: Any
    availability_alloc: Any
    duty_cycle: Any
    env_factor: Any
    goal_measure_id: Any
    hazard_rate_alloc: Any
    hazard_rate_goal: Any
    included: Any
    int_factor: Any
    allocation_method_id: Any
    mission_time: Any
    mtbf_alloc: Any
    mtbf_goal: Any
    n_sub_systems: Any
    n_sub_elements: Any
    parent_id: Any
    percent_weight_factor: Any
    reliability_alloc: Any
    reliability_goal: Any
    op_time_factor: Any
    soa_factor: Any
    weight_factor: Any
    def get_attributes(self): ...



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKProgramStatusRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Any
    __tablename__: str
    __table_args__: Any
    revision_id: Any
    status_id: Any
    cost_remaining: Any
    date_status: Any
    time_remaining: Any
    def get_attributes(self): ...



# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.models.design_mechanic.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK Design Mechanic model package."""



# Standard Library Imports
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKMessageDialog as RAMSTKMessageDialog
from ramstk.views.gtk3.widgets import RAMSTKModuleView as RAMSTKModuleView
from ramstk.views.gtk3.widgets import RAMSTKPanel as RAMSTKPanel
from ramstk.views.gtk3.widgets import RAMSTKWorkView as RAMSTKWorkView

# RAMSTK Local Imports
from . import ValidationTaskDescriptionPanel as ValidationTaskDescriptionPanel
from . import ValidationTaskEffortPanel as ValidationTaskEffortPanel
from . import ValidationTreePanel as ValidationTreePanel

class ValidationModuleView(RAMSTKModuleView):
    _tag: str
    _tablabel: str
    _tabtooltip: str
    _lst_mnu_labels: Any
    _lst_tooltips: Any
    _pnlPanel: Any
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def do_request_delete(self, __button: Gtk.ToolButton) -> None: ...
    _record_id: Any
    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None: ...
    def __make_ui(self) -> None: ...

class ValidationGeneralDataView(RAMSTKWorkView):
    _tag: str
    _tablabel: str
    _tabtooltip: str
    _lst_callbacks: Any
    _lst_icons: Any
    _lst_mnu_labels: Any
    _lst_tooltips: Any
    _pnlTaskDescription: Any
    _pnlTaskEffort: Any
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def _do_request_calculate(self, __button: Gtk.ToolButton) -> None: ...
    def _do_request_calculate_all(self, __button: Gtk.ToolButton) -> None: ...
    _record_id: Any
    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None: ...
    def __make_ui(self) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.program_status.view.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Program Status Views."""

# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
from pandas.plotting import register_matplotlib_converters
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKFrame, RAMSTKPlot, RAMSTKWorkView

# RAMSTK Local Imports
from . import ProgramStatusPlotPanel

register_matplotlib_converters()


class ProgramStatusWorkView(RAMSTKWorkView):
    """Display Verification task burn down curve in the RAMSTK Work Book.

    The Verification burn down Curve displays the planned burn down curve (
    solid line) for all tasks in the V&V plan as well as the actual progress
    (points).  The attributes of a Verification burn down curve view are:

    :cvar _tag: the name of the module.

    :ivar _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag = "validation"
    _tablabel = (
        "<span weight='bold'>" + _("Program\nVerification\nProgress") + "</span>"
    )
    _tabtooltip = _(
        "Shows a plot of the total expected time to complete all verification "
        "tasks and the current progress on those tasks."
    )

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Work View for the Verification package.

        :param configuration: the RAMSTK configuration instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_callbacks = [
            self._do_request_calculate_all,
        ]
        self._lst_icons = [
            "chart",
        ]
        self._lst_mnu_labels = [_("Plot Verification Effort")]
        self._lst_tooltips = [
            _(
                "Plot the overall Verification program plan (i.e., "
                "all Verification tasks) and current status."
            ),
        ]

        # Initialize private scalar attributes.
        self._pnlPanel = ProgramStatusPlotPanel()

        self._title: str = _("Program Verification Effort")

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.pltPlot: RAMSTKPlot = RAMSTKPlot()

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_validation")

        pub.subscribe(self._do_set_cursor_active, "succeed_calculate_verification_plan")

    def _do_request_calculate_all(self, __button: Gtk.ToolButton) -> None:
        """Request to calculate program cost and time.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        :rtype: None
        """
        super().do_set_cursor_busy()
        pub.sendMessage(
            "request_calculate_plan",
        )

    # pylint: disable=unused-argument
    # noinspection PyUnusedLocal
    def _do_set_cursor_active(self, attributes: Dict[str, Any]) -> None:
        """Wrap do_set_cursor_active() of the meta-class.

        This method is called whenever the verification plan is calculated
        successfully.  That PyPubSub MDS includes an attributes data package
        (which is a dict containing the data to plot).  This method is
        needed since the meta-class do_set_cursor_active() method is
        expecting a treelib.Tree() in the MDS.

        :param attributes: the attributes dict for the selected Validation
            task.
        :return: None
        :rtype: None
        """
        super().do_set_cursor_active()

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the Verification task record ID.

        :param attributes: the attributes dict for the selected Validation
            task.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["record_id"] = attributes["validation_id"]

    def __make_ui(self) -> None:
        """Build the user interface for the Verification Status tab.

        :return: None
        :rtype: None
        """
        super().do_make_layout()

        _frame: RAMSTKFrame = RAMSTKFrame()
        _frame.do_set_properties(**{"title": _("Program Verification Effort")})
        _frame.add(self._pnlPanel)

        self.pack_end(_frame, True, True, 0)
        self.show_all()
